/*----------------------------------------------------------------------*\
 | Graphical Sea Ice retrieval program -- Tcl/Tk+C version				|
 |																		|
 | Peter N. Schweitzer (U.S. Geological Survey, Reston, VA 20192)		|
\*----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include <tcl.h>
#include <tk.h>

#define nint(x) ((x - floor(x) < 0.5) ? (int)(x) : (int)(x + 0.5))

static unsigned g_width,g_height;
static char hemisphere = 0;

static void xytoll (int x, int y, double *lat, double *lon) {
	double ax,ay;

	ax = ((double) x) * 25.0 - (((hemisphere == 'N') ? 3850.0 : 3950.0) - 12.5);
	ay = ((double) g_height - y) * 25.0 - (((hemisphere == 'N') ? 5350.0 : 3950.0) - 12.5);
	mapxy (ax,ay,hemisphere,lat,lon);
	}

static void lltoxy (double lat, double lon, int *x, int *y) {
	double ax,ay;

	lon += (hemisphere == 'N') ? 45.0 : 180.0;
	if (lon > 360.0) lon -= 360.0;
	mapll (lat,lon,&ax,&ay);
	*x = nint ((ax + ((hemisphere == 'N') ? 3850.0 : 3950.0) - 12.5)/25.0);
	*y = nint ((ay + ((hemisphere == 'N') ? 5350.0 : 3950.0) - 12.5)/25.0);
	*y = g_height - *y;
	}

/*----------------------------------------------------------------------*\
 | Given pixel coordinates x and y, calculate geographic coordinates	|
\*----------------------------------------------------------------------*/

int geo_of_cmd (ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]) {
	int x,y;
	double latitude,longitude;
	char string [32];
	Tcl_Obj *obj;

	if (argc > 2) {
		x = strtol (argv[1],NULL,0);
		y = strtol (argv[2],NULL,0);
		xytoll (x,y,&latitude,&longitude);

		Tcl_ResetResult (interp);
		obj = Tcl_NewListObj (0,NULL);
		sprintf (string,"%.1lf",longitude);
		Tcl_ListObjAppendElement (interp,obj,Tcl_NewStringObj (string,strlen (string)));
		sprintf (string,"%.1lf",latitude);
		Tcl_ListObjAppendElement (interp,obj,Tcl_NewStringObj (string,strlen (string)));
		Tcl_SetObjResult (interp,obj);
		return (TCL_OK);
		}
	else {
		Tcl_SetResult (interp,"geo_of needs arguments x and y",TCL_STATIC);
		return (TCL_ERROR);
		}
	}

/*----------------------------------------------------------------------*\
 | Given geographic coordinates, calculate pixel coordinates			|
\*----------------------------------------------------------------------*/

int xy_of_cmd (ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]) {
	int x,y;
	double latitude,longitude;
	char string [32];
	Tcl_Obj *obj;

	if (argc > 2) {
		longitude = strtod (argv[1],NULL);
		latitude  = strtod (argv[2],NULL);
		if (hemisphere == 0)
			hemisphere = ((latitude >= 0.0) ? 'N': 'S');
		lltoxy (latitude,longitude,&x,&y);

		Tcl_ResetResult (interp);
		obj = Tcl_NewListObj (0,NULL);
		sprintf (string,"%d",x);
		Tcl_ListObjAppendElement (interp,obj,Tcl_NewStringObj (string,strlen (string)));
		sprintf (string,"%d",y);
		Tcl_ListObjAppendElement (interp,obj,Tcl_NewStringObj (string,strlen (string)));
		Tcl_SetObjResult (interp,obj);
		return (TCL_OK);
		}
	else {
		Tcl_SetResult (interp,"xcoord_of needs arguments longitude and latitude",TCL_STATIC);
		return (TCL_ERROR);
		}
	}

/*----------------------------------------------------------------------*\
 | Store image height and width and hemisphere.							|
\*----------------------------------------------------------------------*/

int set_pole_cmd (ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]) {
	if (argc > 1) {
		char c = toupper (*argv[1]);
		if (c == 'N') {
			g_width = 304;
			g_height = 448;
			hemisphere = c;
			}
		else
			if (c == 'S') {
				g_width = 316;
				g_height = 332;
				hemisphere = c;
				}
			else {
				Tcl_SetResult (interp,"set_pole argument must be either N or S",TCL_STATIC);
				return (TCL_ERROR);
				}
		return (TCL_OK);
		}
	else {
		Tcl_SetResult (interp,"set_ pole needs argument N or S ",TCL_STATIC);
		return (TCL_ERROR);
		}
	}

/*----------------------------------------------------------------------*\
 | Call Tcl_CreateCommand for application-specific commands, if they	|
 | weren't already created by the init procedures called above.			|
\*----------------------------------------------------------------------*/

int Et_AppInit (Tcl_Interp *interp) {
	Tcl_CreateCommand (interp,"geo_of",   geo_of_cmd,   (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	Tcl_CreateCommand (interp,"xy_of",    xy_of_cmd,    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
	Tcl_CreateCommand (interp,"set_pole", set_pole_cmd, (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

	return (TCL_OK);
	}

/*----------------------------------------------------------------------*\
 | Main procedure
\*----------------------------------------------------------------------*/

int main (int argc, char *argv[]) {
	Et_Init (argc,argv);
	return (0);
	}

/*----------------------------------------------------------------------*\
\*----------------------------------------------------------------------*/
